﻿using System;
using System.Threading;
using System.Windows;

namespace TestProject
{
    /// <summary>
    /// 지연 실행 액션
    /// </summary>
    public sealed class DeferredAction : IDisposable
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 타이머
        /// </summary>
        private readonly Timer timer;

        /// <summary>
        /// 리소스 해제 여부
        /// </summary>
        private bool isDisposed;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 생성자 - DeferredAction(action)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="action">액션</param>
        private DeferredAction(Action<DeferredAction> action)
        {
            this.timer = new Timer
            (
                state => Application.Current?.Dispatcher?.Invoke(() => action(this))
            );
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 생성하기 - Create(action)

        /// <summary>
        /// 생성하기
        /// </summary>
        /// <param name="action">액션</param>
        /// <returns>지연 실행 액션</returns>
        public static DeferredAction Create(Action<DeferredAction> action)
        {
            if(action == null)
            {
                throw new ArgumentNullException(nameof(action));
            }

            return new DeferredAction(action);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 지연 실행하기 - Defer(deferTime)

        /// <summary>
        /// 지연 실행하기
        /// </summary>
        /// <param name="deferTime">지연 시간</param>
        public void Defer(TimeSpan deferTime)
        {
            this.timer.Change(deferTime, Timeout.InfiniteTimeSpan);
        }

        #endregion
        #region 리소스 해제하기 - Dispose()

        /// <summary>
        /// 리소스 해제하기
        /// </summary>
        public void Dispose()
        {
            if(this.isDisposed)
            {
                return;
            }

            this.isDisposed = true;

            this.timer.Dispose();
        }

        #endregion
    }
}